/*---------------------------------------------------------------------------*
  Project:  SoftKeyboard
  File:     kbd.h

  Copyright 2005 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  $Log: kbd.h,v $
  Revision 1.2  2005/07/04 01:10:58  hatamoto_minoru
  Comment revisions

  Revision 1.1  2005/06/14 11:30:01  hatamoto_minoru
  Initial registration

  $NoKeywords: $
 *---------------------------------------------------------------------------*/

#ifndef KBD_H_
#define KBD_H_


#include <nitro/types.h>
#include "kbd_autoRepeat.h"
#include "kbd_input.h"
#include "kbd_util.h"

#ifdef __cplusplus
extern "C" {
#endif


// touch event button IDs
// 0x0000 to 0x00ff: for buttons on the keyboard
// 0x0100 to 0xfeff: for touch events that can be added by the application
// 0xff00 to 0xffff: system reserved IDs
#define KBD_BUTTON_ID_NONE 0xff00  // when nothing has been pressed, or when a location with no event has been pressed
#define KBD_BUTTON_ID_KBD_START 0x0000
#define KBD_BUTTON_ID_KBD_END   0x00ff
#define KBD_BUTTON_ID_EXT_START 0x0100
#define KBD_BUTTON_ID_EXT_END   0xfeff
#define KBD_BUTTON_ID_SYS_START 0xff00
#define KBD_BUTTON_ID_SYS_END   0xffff


// keyboard types
typedef enum
{
    KBD_KIND_ASCII,     // ASCII keyboard
    KBD_KIND_JAPANESE,  // Japanese keyboard
    KBD_KIND_EUROPE,    // extended character keyboard
    KBD_KIND_SIGN,      // symbol keyboard
    KBD_KIND_PICTURE,   // pictograph keyboard
    KBD_KIND_NUM
} KBDKind;

// SE types
typedef enum
{
	KBD_SE_SWITCH_LETER,
	KBD_SE_BACKSPACE,
	KBD_SE_KEYINPUT,
	KBD_SE_ENTER,
	KBD_SE_KEYINPUT_EX,
	KBD_SE_BACKSPACE_REL,
	KBD_SE_KEYINPUT_REL,
	KBD_SE_ENTER_REL,
	KBD_SE_KEYINPUT_EX_REL,
	KBD_SE_ERROR,
	KBD_SE_MENU_TP_TAP,
	KBD_SE_LAST
} KBDSoundId;

// Function called for drag&drop. Called within KBD_Update.
// xy are absolute screen coordinates.
typedef void (*KBDDragDropCallback)( void* param, u16 charcode, int x, int y );

// callback function called when a keyboard button event occurs
typedef void (*KBDSoundCallback)( void* param, int sound );

// callback function called when a additional touch event occurs
typedef void (*KBDExtTpEventCallback)( void* param, KBDTpEvent event, u16 pressd_id, u16 current_id, int x, int y );

// touch event button rectangle structure
typedef struct tagKBDButtonRect
{
    u16 x:8;
    u16 y:8;
    u16 width:8;
    u16 height:8;
} KBDButtonRect;

// touch event button structure
typedef struct tagKBDButton
{
    KBDButtonRect rect; // buttom rectangle
    s16 group_offset;   // offset for buttons in the same group
    u16 id;             // event button ID
} KBDButton;


void KBD_Init( int x, int y, BOOL bootmenu_mode, KBDSoundCallback callback, void* param );

void KBD_FillScreen( GXScrText32x32* scr, ///< [out] screen fill
                      int palette, ///< [in] palette number
                      int kbd_chartop ///< [in] keyboard header character number
    );
void KBD_LoadPltt( int palette );

void KBD_SetExtTpEvent( const KBDButton event_list[], u16 num_element, KBDExtTpEventCallback callback, void* param );
    
void KBD_Change( KBDKind kind );
void KBD_ChangeByNumber( int no );
void KBD_ChangeToNext( void );
KBDKind KBD_GetKind( void );

void KBD_Update( void );
void KBD_UpdateValue( u16 paddata, TPData tpdata );

u16  KBD_GetInput( void );
u16  KBD_GetDragging( void );
BOOL KBD_IsUsing( void );
void* KBD_GetCharImg( void );
u32   KBD_GetCharSize( void );
void KBD_GetCursorPos( KBDVec2S16 pos[] );

u16 KBD_ConvertDakuten( u16 c );
u16 KBD_ConvertHandaku( u16 c );
u16 KBD_ConvertSmall( u16 c );
u16 KBD_ConvertAll( u16 c );
BOOL KBD_IsDakuten( u16 c );
BOOL KBD_IsHandaku( u16 c );
BOOL KBD_IsSmall( u16 c );

void KBD_InputWithCursor( void );
void KBD_Redraw(void);

void KBD_SetDragDropCallback( KBDDragDropCallback func, void* param );


#ifdef __cplusplus
}
#endif

#endif // KBD_H_
